@@ -1,6 +1,7 @@
 import gobject
 import glib
 import gtk
+import gnome.gnomevfs as gnomevfs
 import hildondesktop
 import hildon
 import dbus
@@ -27,51 +28,83 @@ class diggItem:
 class diggLoader:
     def __init__(self, callback):
         self.callback = callback
-        self.stream = None
+        self.stream = ""
         self.load_thumbnails = True
+        self.total = 0
+        self.fetched = 0
+        self.result = None
+        self.downresult = None
+        self.inputstream = None
+
+    def stop(self):
+        self.stream = ""
+        self.total = 0
+        self.fetched = 0
+#        if self.result != None:
+#            self.result.complete_in_idle()
+#        if self.downresult != None:
+            #self.downresult.complete_in_idle()
+#            self.inputstream.close()
+#        self.loop.quit()
+
+    def read_and_return(self, filestream, result):
+        print "Read:"
+        print dir(result)
+        self.result = result
+        temp = filestream.read_finish(result)
+        if temp:
+            self.stream += temp
+            self.fetched += len(temp)
+            filestream.read_async(4096, self.read_and_return)
+        else:
+            doc = xmlparser.parseString(self.stream)
+            if doc != None:
+                stories = doc.getElementsByTagName("story")
+                if len(stories) == 0:
+                    return None
+                list = []
+                for story in stories:
+                    title = story.getElementsByTagName("title")[0]
+                    href = story.getElementsByTagName("shorturl")[0]
+                    title = title.childNodes[0].data
+                    href = href.getAttribute("short_url")
+                    diggs = story.getAttribute("diggs")
+                    topic = story.getElementsByTagName("topic")[0]
+                    topic = topic.getAttribute("name")
+                    item = diggItem(title, href, topic, diggs) 
+            #        print thumbnail
+                    if self.load_thumbnails:
+                        thumbnail = None
+                        thumbnail = story.getElementsByTagName("thumbnail")
+                        if thumbnail.length > 0:
+                            thumbnail = thumbnail[0]
+                            thumbnail = thumbnail.getAttribute("src")
+                        try:
+                            if thumbnail != None and thumbnail != "":
+                                g = urllib.urlopen(thumbnail)
+                                img = g.read()
+                                item.thumbnail = img
+                            #print item.thumbnail
+                        except:
+                            item.thumbnail = None
+                    list.append(item)
+                self.callback(list)
+#            result.complete_in_idle()
+            self.stop()
 
     def parse_and_return(self, daemonfile, result):
-        inputstream = self.stream.read_finish(result)
+        print "Parse:"
+        print dir(result)
         try:
-            doc = xmlparser.parseString(inputstream.read())
-        except Exception, e:
-            doc = None
-            print "Can't connect to Digg.com", e
-            self.callback(list=None)
-        finally:
-            inputstream.close()
-            
-        if doc != None:
-            stories = doc.getElementsByTagName("story")
-            if len(stories) == 0:
-                return None
-            list = []
-            for story in stories:
-                title = story.getElementsByTagName("title")[0]
-                href = story.getElementsByTagName("shorturl")[0]
-                title = title.childNodes[0].data
-                href = href.getAttribute("short_url")
-                diggs = story.getAttribute("diggs")
-                topic = story.getElementsByTagName("topic")[0]
-                topic = topic.getAttribute("name")
-                item = diggItem(title, href, topic, diggs) 
-        #        print thumbnail
-                if self.load_thumbnails:
-                    thumbnail = None
-                    thumbnail = story.getElementsByTagName("thumbnail")
-                    if thumbnail.length > 0:
-                        thumbnail = thumbnail[0]
-                        thumbnail = thumbnail.getAttribute("src")
-                    try:
-                        if thumbnail != None and thumbnail != "":
-                            g = urllib.urlopen(thumbnail)
-                            img = g.read()
-                            item.thumbnail = img
-                        #print item.thumbnail
-                    except:
-                        item.thumbnail = None
-                list.append(item)
-            self.callback(list)
+            self.inputstream = daemonfile.read_finish(result)
+        except gio.Error, e:
+            self.stop()
+            return
+        info = daemonfile.query_info(gio.FILE_ATTRIBUTE_STANDARD_SIZE)
+        self.total = info.get_attribute_uint64(gio.FILE_ATTRIBUTE_STANDARD_SIZE)
+        self.inputstream.read_async(4096, self.read_and_return)
+        self.downresult = result
+
 
 
     def get_toplist(self, container="", topic="", count=3, hotness="top"):
@@ -83,8 +116,9 @@ class diggLoader:
             selected = "container/" + container + "/"
         params = urllib.urlencode({"count":count, "appkey":"http://maemo.org/packages/view/digghomewidget/"})
         try:
-            self.stream = gio.File("http://services.digg.com/stories/"+selected+hotness+"?%s" % params)
-            self.stream.read_async(self.parse_and_return)
+            f = gio.File("http://services.digg.com/stories/"+selected+hotness+"?%s" % params)
+            inputstream = f.read()
+            inputstream.read_async(4096, self.read_and_return)
         except Exception, e:
             print "Can't connect to Digg.com", e
 
@@ -406,37 +440,6 @@ class DiggHomePlugin(hildondesktop.HomePluginItem):
             hildon.hildon_banner_show_information(self, "None", "Can't connect to digg.com. Please check your internet connection.")
 
 
-    def category_changed(self, button, user_data=None):
-        value = button.get_value()
-        value = value.replace("&", "_")
-        if value == "All":
-            value = ""
-        self.category = value
-        self.row.title = "Updating..."
-        self.row.queue_draw()
-        self.loader.get_toplist(container=self.category,count=self.count)
-#        glib.timeout_add(50, self.update)
-#        self.update()
-
-        
-
-    def show_selector(self):
-        selector = hildon.TouchSelector(text = True) 
-        selector.connect("changed", self.selection_changed)
-        selector.append_text("All")
-        selector.append_text("Technology")
-        selector.append_text("World&Business")
-        selector.append_text("Entertainment")
-        selector.append_text("Gaming")
-        selector.append_text("Science")
-        selector.append_text("Sports")
-        selector.append_text("Lifestyle")
-        selector.append_text("Offbeat")
-        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
-        return selector
-       # pass
-#        widget.set_state(gtk.STATE_NORMAL)
-#        widget.queue_draw()
     def selection_changed(self, selector, user_data):
         current_selection = selector.get_current_text()
 #        print current_selection
